<?php

class Paygenius_Common_Model_Client_Paygenius extends Varien_Object
{
    private $config;

    public function init($config)
    {
        $this->config = $config;

        return $this;
    }

    public function paymentCreate($number, $holder, $year, $month, $cvv, $type, $reference, $currency, $amount,
                                  $firstname, $lastname, $email)
    {
        return $this->__send('payment/create',
                array(
                'creditCard' => array(
                    'number' => $number,
                    'cardHolder' => $holder,
                    'expiryYear' => $year,
                    'expiryMonth' => $month,
                    'cvv' => $cvv,
                    'type' => $type
                ),
                'consumer' => array(
                    'name' => $firstname,
                    'surname' => $lastname,
                    'email' => $email
                ),
                'transaction' => array(
                    'reference' => $reference,
                    'currency' => $currency,
                    'description' => '',
                    'amount' => $amount * 100
                ),
                'threeDSecure' => false
        ));
    }

    public function confirm3DS($reference, $pares)
    {
        return $this->__send(sprintf('payment/%s/confirm', $reference),
                array(
                'paRes' => $pares
        ));
    }

    public function execute($reference)
    {
        return $this->__send(sprintf('payment/%s/execute', $reference));
    }

    public function redirectCreate($name, $surname, $amount, $currency, $email, $reference, $page, $returnUrl)
    {
        return $this->__send('redirect/create',
                array(
                'transaction' => array(
                    'reference' => $reference,
                    'currency' => $currency,
                    'amount' => $amount
                ),
                'consumer' => array(
                    'name' => $name,
                    'surname' => $surname,
                    'email' => $email
                ),
                'urls' => array(
                    'cancel' => $returnUrl,
                    'error' => $returnUrl,
                    'success' => $returnUrl
                ),
                'page' => $page
        ));
    }

    public function payment($reference)
    {
        return $this->__send(sprintf('payment/%s', $reference));
    }

    public function pageList()
    {
        return $this->__send('page/list');
    }

    private function isTestMode()
    {
        return $this->getConfig('test') == 1;
    }

    private function getEndPoint()
    {
        if ($this->isTestMode()) {
            return Paygenius_Common_Model_Config::ENDPOINT_TEST;
        } else {
            return Paygenius_Common_Model_Config::ENDPOINT_LIVE;
        }
    }

    private function getConfig($key)
    {
        return Mage::getStoreConfig(sprintf('payment/%s/%s', $this->config, $key));
    }

    private function __send($endPoint, $data = null)
    {
        if ($data !== null) {
            $data = Mage::helper('core')->jsonEncode($data);
        }

        $url = sprintf('%s%s', $this->getEndPoint(), $endPoint);

        $client = new Varien_Http_Client($url);
        $client->setHeaders('X-Token', $this->getConfig('token'));
        $client->setHeaders('X-Signature', $this->sign($url, $data == null ? '' : $data));

        if ($data === null) {
            $client->setMethod(Varien_Http_Client::GET);
        } else {
            $client->setMethod(Varien_Http_Client::POST);
            $client->setRawData($data, 'application/json');
        }

        $response = $client->request();

        $body = $response->getBody();

        return Mage::helper('core')->jsonDecode($body);
    }

    private function sign($endpoint, $request = '')
    {
        $toSign = trim($endpoint."\n".$request);

        return hash_hmac('sha256', $toSign, $this->getConfig('secret'));
    }
}